@@ -2,27 +2,45 @@
 dtnorm <- function(x, mean = 0, sd = 1, left = -Inf, right = Inf, log = FALSE) {
   input <- data.frame(x = as.numeric(x), mean = as.numeric(mean), sd = as.numeric(sd), 
     left = as.numeric(left), right = as.numeric(right))
-  with(input, .Call("dtnorm", x, mean, sd, left, right, log))
+  rval <- with(input, .Call("cdtnorm", x, mean, sd, left, right, log))
+  if(is.matrix(x)) {
+    rval <- matrix(rval, ncol = ncol(x), nrow = nrow(x))
+    colnames(rval) <- colnames(x)
+    rownames(rval) <- rownames(x)
+  }
+  return(rval)
 }
 
 
 ## distribution function
-ptnorm <- function(q, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE, 
-  left = -Inf, right = Inf) {
+ptnorm <- function(q, mean = 0, sd = 1, left = -Inf, right = Inf, 
+  lower.tail = TRUE, log.p = FALSE) {
   input <- data.frame(q = as.numeric(q), mean = as.numeric(mean), sd = as.numeric(sd), 
     left = as.numeric(left), right = as.numeric(right))
-  with(input, .Call("ptnorm", q, mean, sd, left, right, lower.tail, log.p))
+  rval <- with(input, .Call("cptnorm", q, mean, sd, left, right, lower.tail, log.p))
+  if(is.matrix(q)) {
+    rval <- matrix(rval, ncol = ncol(q), nrow = nrow(q))
+    colnames(rval) <- colnames(q)
+    rownames(rval) <- rownames(q)
+  }
+  return(rval)
 }
 
 ## quantiles
-qtnorm <- function(p, mean = 0, sd = 1, lower.tail = TRUE, log.p = FALSE, 
-  left = -Inf, right = Inf) {
+qtnorm <- function(p, mean = 0, sd = 1, left = -Inf, right = Inf,
+  lower.tail = TRUE, log.p = FALSE) {
   if(log.p) p <- exp(p) 
   lower <- if(lower.tail) left else right
   upper <- if(lower.tail) right else left
   p <- pnorm((lower-mean)/sd, lower.tail = lower.tail) * (1 - p) + 
     p*pnorm((upper - mean)/sd, lower.tail = lower.tail)
-  qnorm(p, lower.tail = lower.tail)*sd + mean
+  rval <- qnorm(p, lower.tail = lower.tail)*sd + mean
+  if(is.matrix(p)) {
+    rval <- matrix(rval, ncol = ncol(p), nrow = nrow(p))
+    colnames(rval) <- colnames(p)
+    rownames(rval) <- rownames(p)
+  }
+  return(rval)
 }
 
 ## random numbers
@@ -33,8 +51,8 @@ rtnorm <- function(n, mean = 0, sd = 1, left = -Inf, right = Inf) {
 
 
 ## scores
-stnorm <- function(x, mean = 0, sd = 1, which = c("mu", "sigma"), 
-  left = -Inf, right = Inf) {
+stnorm <- function(x, mean = 0, sd = 1, left = -Inf, right = Inf,
+  which = c("mu", "sigma")) {
   input <- data.frame(x = as.numeric(x), mean = as.numeric(mean), sd = as.numeric(sd), 
     left = as.numeric(left), right = as.numeric(right))
   if(!is.character(which))
@@ -56,8 +74,8 @@ stnorm <- function(x, mean = 0, sd = 1, which = c("mu", "sigma"),
 }
 
 ## Hessian
-htnorm <- function(x, mean = 0, sd = 1, which = c("mu", "sigma"), 
-  left = -Inf, right = Inf) {
+htnorm <- function(x, mean = 0, sd = 1, left = -Inf, right = Inf,
+  which = c("mu", "sigma")) {
   input <- data.frame(x = as.numeric(x), mean = as.numeric(mean), sd = as.numeric(sd), 
     left = as.numeric(left), right = as.numeric(right))
   if(!is.character(which))
@@ -80,3 +98,13 @@ htnorm <- function(x, mean = 0, sd = 1, which = c("mu", "sigma"),
   colnames(hess)[colnames(hess) == "dsigma"] <- "d2sigma"
   hess
 }
+
+
+## Expectation
+etnorm <- function(mean = 0, sd = 1, left = -Inf, right = Inf) {
+  rmm <- (right-mean)/sd
+  lmm <- (left-mean)/sd
+  pncens <- pnorm(rmm)-pnorm(lmm)
+  rval <- mean + sd*(dnorm(lmm) - dnorm(rmm))/pncens
+  rval
+}
